@@ -3,6 +3,7 @@
    Data Link Provider Interface (DLPI) network interface code. */
 
 /*
+ * Copyright (c) 2009-2011,2014 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 2004,2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
@@ -22,11 +23,11 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
  * This software was written for Internet Systems Consortium
  * by Eric James Negaard, <lmdejn@lmd.ericsson.se>.  To learn more about
- * Internet Systems Consortium, see ``http://www.isc.org''.
+ * Internet Systems Consortium, see ``https://www.isc.org''.
  *
  * Joost Mulders has also done considerable work in debugging the DLPI API
  * support on Solaris and getting this code to work properly on a variety
@@ -87,7 +88,8 @@
 
 #include "dhcpd.h"
 
-#if defined (USE_DLPI_SEND) || defined (USE_DLPI_RECEIVE)
+#if defined (USE_DLPI_SEND) || defined (USE_DLPI_RECEIVE) || \
+    defined(USE_DLPI_HWADDR)
 
 # include <sys/ioctl.h>
 # include <sys/time.h>
@@ -123,7 +125,7 @@
 # endif
 
 #if defined(USE_DLPI_PFMOD) || defined(USE_DLPI_RAW)
-static int strioctl PROTO ((int fd, int cmd, int timeout, int len, char *dp));
+static int strioctl (int fd, int cmd, int timeout, int len, char *dp);
 #endif
 
 #define DLPI_MAXDLBUF		8192	/* Buffer size */
@@ -131,42 +133,44 @@ static int strioctl PROTO ((int fd, int cmd, int timeout, int len, char *dp));
 #define DLPI_DEVDIR		"/dev/"	/* Device directory */
 
 static int dlpiopen(const char *ifname);
-static int dlpiunit PROTO ((char *ifname));
-static int dlpiinforeq PROTO ((int fd));
-static int dlpiphysaddrreq PROTO ((int fd, unsigned long addrtype));
-static int dlpiattachreq PROTO ((int fd, unsigned long ppa));
-static int dlpibindreq PROTO ((int fd, unsigned long sap, unsigned long max_conind,
-			       unsigned long service_mode, unsigned long conn_mgmt,
-			       unsigned long xidtest));
+static int dlpiunit (char *ifname);
+static int dlpiinforeq (int fd);
+static int dlpiphysaddrreq (int fd, unsigned long addrtype);
+static int dlpiattachreq (int fd, unsigned long ppa);
+static int dlpibindreq (int fd, unsigned long sap, unsigned long max_conind,
+			unsigned long service_mode, unsigned long conn_mgmt,
+			unsigned long xidtest);
 #if defined(UNUSED_DLPI_INTERFACE)
 /* These functions are unused at present, but may be used at a later date.
  * defined out to avoid compiler warnings about unused static functions.
  */
-static int dlpidetachreq PROTO ((int fd));
-static int dlpiunbindreq PROTO ((int fd));
+static int dlpidetachreq (int fd);
+static int dlpiunbindreq (int fd);
 #endif
-static int dlpiokack PROTO ((int fd, char *bufp));
-static int dlpiinfoack PROTO ((int fd, char *bufp));
-static int dlpiphysaddrack PROTO ((int fd, char *bufp));
-static int dlpibindack PROTO ((int fd, char *bufp));
-static int dlpiunitdatareq PROTO ((int fd, unsigned char *addr,
-				   int addrlen, unsigned long minpri,
-				   unsigned long maxpri, unsigned char *data,
-				   int datalen));
-static int dlpiunitdataind PROTO ((int fd,
-				   unsigned char *dstaddr,
-				   unsigned long *dstaddrlen,
-				   unsigned char *srcaddr,
-				   unsigned long *srcaddrlen,
-				   unsigned long *grpaddr,
-				   unsigned char *data,
-				   int datalen));
-
-static int	expected PROTO ((unsigned long prim, union DL_primitives *dlp,
-				  int msgflags));
-static int	strgetmsg PROTO ((int fd, struct strbuf *ctlp,
-				  struct strbuf *datap, int *flagsp,
-				  char *caller));
+static int dlpiokack (int fd, char *bufp);
+static int dlpiinfoack (int fd, char *bufp);
+static int dlpiphysaddrack (int fd, char *bufp);
+static int dlpibindack (int fd, char *bufp);
+#if defined(USE_DLPI_SEND) || defined(USE_DLPI_RECEIVE)
+/* These functions are not used if we're only sourcing the get_hw_addr()
+ * function (for USE_SOCKETS).
+ */
+static int dlpiunitdatareq (int fd, unsigned char *addr, int addrlen, 
+			    unsigned long minpri, unsigned long maxpri, 
+			    unsigned char *data, int datalen);
+static int dlpiunitdataind (int fd,
+			    unsigned char *dstaddr,
+			    unsigned long *dstaddrlen,
+			    unsigned char *srcaddr,
+			    unsigned long *srcaddrlen,
+			    unsigned long *grpaddr,
+			    unsigned char *data,
+			    int datalen);
+#endif /* !USE_DLPI_HWADDR: USE_DLPI_SEND || USE_DLPI_RECEIVE */
+static int expected (unsigned long prim, union DL_primitives *dlp, 
+		     int msgflags);
+static int strgetmsg (int fd, struct strbuf *ctlp, struct strbuf *datap,
+		      int *flagsp, char *caller);
 
 /* Reinitializes the specified interface after an address change.   This
    is not required for packet-filter APIs. */
@@ -204,7 +208,6 @@ int if_register_dlpi (info)
 	    log_fatal ("Can't open DLPI device for %s: %m", info -> name);
 	}
 
-
 	/*
 	 * Submit a DL_INFO_REQ request, to find the dl_mac_type and 
          * dl_provider_style
@@ -292,8 +295,6 @@ int if_register_dlpi (info)
 	}
 #endif
 
-	get_hw_addr(info->name, &info->hw_address);
-
 	return sock;
 }
 
@@ -525,6 +526,7 @@ ssize_t send_packet (interface, packet, raw, len, from, to, hto)
 {
 #ifdef USE_DLPI_RAW
 	double hh [32];
+	int fudge;
 #endif
 	double ih [1536 / sizeof (double)];
 	unsigned char *dbuf = (unsigned char *)ih;
@@ -532,12 +534,14 @@ ssize_t send_packet (interface, packet, raw, len, from, to, hto)
 	unsigned char dstaddr [DLPI_MAXDLADDR];
 	unsigned addrlen;
 	int result;
-	int fudge;
 
 	if (!strcmp (interface -> name, "fallback"))
 		return send_fallback (interface, packet, raw,
 				      len, from, to, hto);
 
+	if (hto == NULL && interface->anycast_mac_addr.hlen)
+		hto = &interface->anycast_mac_addr;
+
 	dbuflen = 0;
 
 	/* Assemble the headers... */
@@ -548,8 +552,6 @@ ssize_t send_packet (interface, packet, raw, len, from, to, hto)
 	fudge = dbuflen % 4; /* IP header must be word-aligned. */
 	memcpy (dbuf + fudge, (unsigned char *)hh, dbuflen);
 	dbuflen += fudge;
-#else
-	fudge = 0;
 #endif
 	assemble_udp_ip_header (interface, dbuf, &dbuflen, from.s_addr,
 				to -> sin_addr.s_addr, to -> sin_port,
@@ -593,7 +595,7 @@ ssize_t send_packet (interface, packet, raw, len, from, to, hto)
           else 
              memcpy ( phys, interface -> dlpi_broadcast_addr.hbuf, 
               interface -> dlpi_broadcast_addr.hlen);
-           
+
           if (sap_len < 0) { 
              memcpy ( dstaddr, phys, phys_len);
              memcpy ( (char *) &dstaddr [phys_len], sap, ABS (sap_len));
@@ -639,6 +641,7 @@ ssize_t receive_packet (interface, buf, len, from, hfrom)
 #endif
 
 	if (length <= 0) {
+	    log_error("receive_packet: %m");
 	    return length;
 	}
 
@@ -662,8 +665,7 @@ ssize_t receive_packet (interface, buf, len, from, hfrom)
               memcpy ((char *) &hfrom -> hbuf [1], srcaddr, phys_len);
             }
             else {
-              memcpy ((char *) &hfrom -> hbuf [1], (char *) &srcaddr [phys_len],
-                phys_len);
+              memcpy((char *)&hfrom->hbuf[1], srcaddr + sap_len, phys_len);
             }
           } 
           else if (hfrom) {
@@ -689,11 +691,18 @@ ssize_t receive_packet (interface, buf, len, from, hfrom)
 	length -= offset;
 #endif
 	offset = decode_udp_ip_header (interface, dbuf, bufix,
-				       from, length, &paylen);
+				       from, length, &paylen, 1);
 
-	/* If the IP or UDP checksum was bad, skip the packet... */
+	/*
+	 * If the IP or UDP checksum was bad, skip the packet...
+	 *
+	 * Note: this happens all the time when writing packets via the
+	 * fallback socket.  The packet received by streams does not have
+	 * the IP or UDP checksums filled in, as those are calculated by
+	 * the hardware.
+	 */
 	if (offset < 0) {
-	    return 0;
+		return 0;
 	}
 
 	bufix += offset;
@@ -969,7 +978,7 @@ static int dlpibindack (fd, bufp)
 	
 	dlp = (union DL_primitives *)ctl.buf;
 	
-	if (!expected (DL_BIND_ACK, dlp, flags) < 0) {
+	if (expected (DL_BIND_ACK, dlp, flags) == -1) {
 		return -1;
 	}
 	
@@ -1003,7 +1012,7 @@ static int dlpiokack (fd, bufp)
 	
 	dlp = (union DL_primitives *)ctl.buf;
 	
-	if (!expected (DL_OK_ACK, dlp, flags) < 0) {
+	if (expected (DL_OK_ACK, dlp, flags) == -1) {
 		return -1;
 	}
 	
@@ -1037,7 +1046,7 @@ static int dlpiinfoack (fd, bufp)
 	
 	dlp = (union DL_primitives *) ctl.buf;
 	
-	if (!expected (DL_INFO_ACK, dlp, flags) < 0) {
+	if (expected (DL_INFO_ACK, dlp, flags) == -1) {
 		return -1;
 	}
 	
@@ -1071,7 +1080,7 @@ int dlpiphysaddrack (fd, bufp)
 
 	dlp = (union DL_primitives *)ctl.buf;
 	
-	if (!expected (DL_PHYS_ADDR_ACK, dlp, flags) < 0) {
+	if (expected (DL_PHYS_ADDR_ACK, dlp, flags) == -1) {
 		return -1;
 	}
 
@@ -1083,6 +1092,7 @@ int dlpiphysaddrack (fd, bufp)
 	return 0;
 }
 
+#if defined(USE_DLPI_SEND) || defined(USE_DLPI_RECEIVE)
 int dlpiunitdatareq (fd, addr, addrlen, minpri, maxpri, dbuf, dbuflen)
 	int fd;
 	unsigned char *addr;
@@ -1144,22 +1154,23 @@ static int dlpiunitdataind (fd, daddr, daddrlen,
 	ctl.maxlen = DLPI_MAXDLBUF;
 	ctl.len = 0;
 	ctl.buf = (char *)buf;
-	
+
 	data.maxlen = dlen;
 	data.len = 0;
 	data.buf = (char *)dbuf;
-	
+
 	result = getmsg (fd, &ctl, &data, &flags);
-	
-	if (result != 0) {
+
+	if (result < 0) {
+		log_debug("dlpiunitdataind: %m");
 		return -1;
 	}
-	
+
 	if (ctl.len < sizeof (dl_unitdata_ind_t) ||
 	    dlp -> unitdata_ind.dl_primitive != DL_UNITDATA_IND) {
 		return -1;
 	}
-	
+
 	if (data.len <= 0) {
 		return data.len;
 	}
@@ -1183,13 +1194,14 @@ static int dlpiunitdataind (fd, daddr, daddrlen,
 	if (daddrlen) {
 		*daddrlen = dlp -> unitdata_ind.dl_dest_addr_length;
 	}
-	
+
 	if (grpaddr) {
 		*grpaddr = dlp -> unitdata_ind.dl_group_address;
 	}
-	
+
 	return data.len;
 }
+#endif /* !USE_DLPI_HWADDR: USE_DLPI_RECEIVE || USE_DLPI_SEND */
 
 /*
  * expected - see if we got what we wanted.
@@ -1201,15 +1213,15 @@ static int expected (prim, dlp, msgflags)
 {
 	if (msgflags != RS_HIPRI) {
 		/* Message was not M_PCPROTO */
-		return 0;
+		return -1;
 	}
 
-	if (dlp -> dl_primitive != prim) {
+	if (dlp->dl_primitive != prim) {
 		/* Incorrect/unexpected return message */
-		return 0;
+		return -1;
 	}
 	
-	return 1;
+	return 0;
 }
 
 /*
@@ -1281,6 +1293,7 @@ static int strgetmsg (fd, ctlp, datap, flagsp, caller)
 	return 0;
 }
 
+#if defined(USE_DLPI_SEND)
 int can_unicast_without_arp (ip)
 	struct interface_info *ip;
 {
@@ -1314,10 +1327,11 @@ void maybe_setup_fallback ()
 		interface_dereference (&fbi, MDL);
 	}
 }
+#endif /* USE_DLPI_SEND */
 
 void 
 get_hw_addr(const char *name, struct hardware *hw) {
-	int sock;
+	int sock, unit;
 	long buf[DLPI_MAXDLBUF];
         union DL_primitives *dlp;
 
@@ -1357,6 +1371,20 @@ get_hw_addr(const char *name, struct hardware *hw) {
 				  (unsigned long)dlp->info_ack.dl_mac_type);
 	}
 
+	if (dlp->info_ack.dl_provider_style == DL_STYLE2) {
+		/*
+		 * Attach to the device.  If this fails, the device
+		 * does not exist.
+		 */
+		unit = dlpiunit((char *)name);
+
+		if (dlpiattachreq(sock, unit) < 0 ||
+		    dlpiokack(sock, (char *)buf) < 0) {
+			log_fatal("Can't attach DLPI device for %s: %m",
+				  name);
+		}
+	}
+
 	/*
 	 * Submit a DL_PHYS_ADDR_REQ request, to find
 	 * the hardware address.
@@ -1383,4 +1411,4 @@ get_hw_addr(const char *name, struct hardware *hw) {
 
 	close(sock);
 }
-#endif /* USE_DLPI */
+#endif /* USE_DLPI_SEND || USE_DLPI_RECEIVE || USE_DLPI_HWADDR */
