@@ -3,7 +3,7 @@
    The dhcpctl remote object. */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004,2007,2009,2014 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -24,19 +24,9 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
-#ifndef lint
-static char copyright[] =
-"$Id: remote.c,v 1.14.140.1 2009/07/23 21:43:34 sar Exp $ Copyright (c) 2004 Internet Systems Consortium.  All rights reserved.\n";
-#endif /* not lint */
-
+#include "dhcpd.h"
 #include <omapip/omapip_p.h>
 #include "dhcpctl.h"
 
@@ -122,8 +112,8 @@ dhcpctl_status dhcpctl_new_object (dhcpctl_handle *h,
 	isc_result_t status;
 
 	m = (dhcpctl_remote_object_t *)0;
-	status = omapi_object_allocate ((omapi_object_t **)&m,
-					dhcpctl_remote_type, 0, MDL);
+	status = omapi_object_allocate((omapi_object_t **)&m,
+				       dhcpctl_remote_type, 0, MDL);
 	if (status != ISC_R_SUCCESS)
 		return status;
 
@@ -187,7 +177,7 @@ dhcpctl_status dhcpctl_open_object (dhcpctl_handle h,
 	dhcpctl_remote_object_t *remote;
 
 	if (h -> type != dhcpctl_remote_type)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	remote = (dhcpctl_remote_object_t *)h;
 
 	status = omapi_message_new (&message, MDL);
@@ -268,7 +258,7 @@ isc_result_t dhcpctl_remote_set_value (omapi_object_t *h,
 	isc_result_t status;
 
 	if (h -> type != dhcpctl_remote_type)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	ro = (dhcpctl_remote_object_t *)h;
 
 	if (!omapi_ds_strcmp (name, "remote-handle")) {
@@ -290,7 +280,7 @@ isc_result_t dhcpctl_remote_get_value (omapi_object_t *h,
 				       omapi_value_t **value)
 {
 	if (h -> type != dhcpctl_remote_type)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	
 	if (h -> inner && h -> inner -> type -> get_value)
 		return (*(h -> inner -> type -> get_value))
@@ -305,7 +295,7 @@ isc_result_t dhcpctl_remote_signal_handler (omapi_object_t *o,
 	omapi_typed_data_t *tv;
 
 	if (o -> type != dhcpctl_remote_type)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	p = (dhcpctl_remote_object_t *)o;
 
 	if (!strcmp (name, "updated")) {
@@ -336,7 +326,7 @@ isc_result_t dhcpctl_remote_destroy (omapi_object_t *h,
 {
 	dhcpctl_remote_object_t *p;
 	if (h -> type != dhcpctl_remote_type)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	p = (dhcpctl_remote_object_t *)h;
 	if (p -> handle)
 		omapi_object_dereference ((omapi_object_t **)&p -> handle,
@@ -354,10 +344,8 @@ isc_result_t dhcpctl_remote_stuff_values (omapi_object_t *c,
 					  omapi_object_t *id,
 					  omapi_object_t *p)
 {
-	int i;
-
 	if (p -> type != dhcpctl_remote_type)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (p -> inner && p -> inner -> type -> stuff_values)
 		return (*(p -> inner -> type -> stuff_values)) (c, id,
