@@ -3,7 +3,8 @@
    Persistent database management routines for DHCPD... */
 
 /*
- * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2012-2015 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2010 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -24,22 +25,16 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
-#ifndef lint
-static char copyright[] =
-"$Id: db.c,v 1.74.32.5 2009/07/23 21:43:35 sar Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
-#endif /* not lint */
-
 #include "dhcpd.h"
 #include <ctype.h>
-#include "version.h"
+#include <errno.h>
+
+#define LEASE_REWRITE_PERIOD 3600
+
+static isc_result_t write_binding_scope(FILE *db_file, struct binding *bnd,
+					char *prepend);
 
 FILE *db_file;
 
@@ -48,13 +43,63 @@ static int count = 0;
 TIME write_time;
 int lease_file_is_corrupt = 0;
 
+/* Write a single binding scope value in parsable format.
+ */
+
+static isc_result_t
+write_binding_scope(FILE *db_file, struct binding *bnd, char *prepend) {
+	char *s;
+
+	if ((db_file == NULL) || (bnd == NULL) || (prepend == NULL))
+		return DHCP_R_INVALIDARG;
+
+	if (bnd->value->type == binding_data) {
+		if (bnd->value->value.data.data != NULL) {
+			s = quotify_buf(bnd->value->value.data.data,
+					bnd->value->value.data.len, MDL);
+			if (s != NULL) {
+				errno = 0;
+				fprintf(db_file, "%sset %s = \"%s\";",
+					prepend, bnd->name, s);
+				dfree(s, MDL);
+				if (errno)
+					return ISC_R_FAILURE;
+			} else {
+			    return ISC_R_FAILURE;
+			}
+		}
+	} else if (bnd->value->type == binding_numeric) {
+		errno = 0;
+		fprintf(db_file, "%sset %s = %%%ld;", prepend,
+			bnd->name, bnd->value->value.intval);
+		if (errno)
+			return ISC_R_FAILURE;
+	} else if (bnd->value->type == binding_boolean) {
+		errno = 0;
+		fprintf(db_file, "%sset %s = %s;", prepend, bnd->name,
+			bnd->value->value.intval ? "true" : "false");
+		if (errno)
+			return ISC_R_FAILURE;
+	} else if (bnd->value->type == binding_dns) {
+		log_error("%s: persistent dns values not supported.",
+			  bnd->name);
+	} else if (bnd->value->type == binding_function) {
+		log_error("%s: persistent functions not supported.",
+			  bnd->name);
+	} else {
+		log_fatal("%s: unknown binding type %d", bnd->name,
+			  bnd->value->type);
+	}
+
+	return ISC_R_SUCCESS;
+}
+
 /* Write the specified lease to the current lease database file. */
 
 int write_lease (lease)
 	struct lease *lease;
 {
 	int errors = 0;
-	int i;
 	struct binding *b;
 	char *s;
 	const char *tval;
@@ -71,7 +116,7 @@ int write_lease (lease)
 	fprintf (db_file, "lease %s {", piaddr (lease -> ip_addr));
 	if (errno) {
 		++errors;
-        }
+	}
 
 	if (lease->starts &&
 	    ((tval = print_time(lease->starts)) == NULL ||
@@ -119,6 +164,20 @@ int write_lease (lease)
 			  : "abandoned")) < 0)
                         ++errors;
 
+	/*
+	 * In this case, if the rewind state is not present in the lease file,
+	 * the reader will use the current binding state as the most
+	 * conservative (safest) state.  So if the in-memory rewind state is
+	 * for some reason invalid, the best thing to do is not to write a
+	 * state and let the reader take on a safe state.
+	 */
+	if ((lease->binding_state != lease->rewind_binding_state) &&
+	    (lease->rewind_binding_state > 0) &&
+	    (lease->rewind_binding_state <= FTS_LAST) &&
+	    (fprintf(db_file, "\n  rewind binding state %s;",
+		     binding_state_names[lease->rewind_binding_state-1])) < 0)
+			++errors;
+
 	if (lease->flags & RESERVED_LEASE)
 		if (fprintf(db_file, "\n  reserved;") < 0)
                         ++errors;
@@ -148,7 +207,6 @@ int write_lease (lease)
 			++errors;
 	}
 	if (lease -> uid_len) {
-		int i;
 		s = quotify_buf (lease -> uid, lease -> uid_len, MDL);
 		if (s) {
 			errno = 0;
@@ -159,49 +217,17 @@ int write_lease (lease)
 		} else
 			++errors;
 	}
-	if (lease -> scope) {
-	    for (b = lease -> scope -> bindings; b; b = b -> next) {
-		if (!b -> value)
+
+	if (lease->scope != NULL) {
+	    for (b = lease->scope->bindings; b; b = b->next) {
+		if (!b->value)
 			continue;
-		if (b -> value -> type == binding_data) {
-		    if (b -> value -> value.data.data) {
-			s = quotify_buf (b -> value -> value.data.data,
-					 b -> value -> value.data.len, MDL);
-			if (s) {
-			    errno = 0;
-			    fprintf (db_file, "\n  set %s = \"%s\";",
-				     b -> name, s);
-			    if (errno)
-				++errors;
-			    dfree (s, MDL);
-			} else
-			    ++errors;
-		    }
-		} else if (b -> value -> type == binding_numeric) {
-		    errno = 0;
-		    fprintf (db_file, "\n  set %s = %%%ld;",
-			     b -> name, b -> value -> value.intval);
-		    if (errno)
+
+		if (write_binding_scope(db_file, b, "\n  ") != ISC_R_SUCCESS)
 			++errors;
-		} else if (b -> value -> type == binding_boolean) {
-		    errno = 0;
-		    fprintf (db_file, "\n  set %s = %s;",
-			     b -> name,
-			     b -> value -> value.intval ? "true" : "false");
-		    if (errno)
-			    ++errors;
-		} else if (b -> value -> type == binding_dns) {
-			log_error ("%s: persistent dns values not supported.",
-				   b -> name);
-		} else if (b -> value -> type == binding_function) {
-			log_error ("%s: persistent functions not supported.",
-				   b -> name);
-		} else {
-			log_error ("%s: unknown binding type %d",
-				   b -> name, b -> value -> type);
-		}
 	    }
 	}
+
 	if (lease -> agent_options) {
 	    struct option_cache *oc;
 	    struct data_string ds;
@@ -222,7 +248,7 @@ int write_lease (lease)
 	    }
 	}
 	if (lease -> client_hostname &&
-	    db_printable (lease -> client_hostname)) {
+	    db_printable((unsigned char *)lease->client_hostname)) {
 		s = quotify_string (lease -> client_hostname, MDL);
 		if (s) {
 			errno = 0;
@@ -233,21 +259,22 @@ int write_lease (lease)
 		} else
 			++errors;
 	}
-	if (lease -> on_expiry) {
+	if (lease->on_star.on_expiry) {
 		errno = 0;
 		fprintf (db_file, "\n  on expiry%s {",
-			 lease -> on_expiry == lease -> on_release
+			 lease->on_star.on_expiry == lease->on_star.on_release
 			 ? " or release" : "");
-		write_statements (db_file, lease -> on_expiry, 4);
+		write_statements (db_file, lease->on_star.on_expiry, 4);
 		/* XXX */
 		fprintf (db_file, "\n  }");
 		if (errno)
 			++errors;
 	}
-	if (lease -> on_release && lease -> on_release != lease -> on_expiry) {
+	if (lease->on_star.on_release &&
+	    lease->on_star.on_release != lease->on_star.on_expiry) {
 		errno = 0;
 		fprintf (db_file, "\n  on release {");
-		write_statements (db_file, lease -> on_release, 4);
+		write_statements (db_file, lease->on_star.on_release, 4);
 		/* XXX */
 		fprintf (db_file, "\n  }");
 		if (errno)
@@ -281,7 +308,7 @@ int write_host (host)
 		if (!new_lease_file ())
 			return 0;
 
-	if (!db_printable (host -> name))
+	if (!db_printable((unsigned char *)host->name))
 		return 0;
 
 	if (counting)
@@ -419,7 +446,6 @@ int write_group (group)
 	struct group_object *group;
 {
 	int errors = 0;
-	int i;
 
 	/* If the lease file is corrupt, don't try to write any more leases
 	   until we've written a good lease file. */
@@ -427,7 +453,7 @@ int write_group (group)
 		if (!new_lease_file ())
 			return 0;
 
-	if (!db_printable (group -> name))
+	if (!db_printable((unsigned char *)group->name))
 		return 0;
 
 	if (counting)
@@ -481,10 +507,238 @@ int write_group (group)
 	return !errors;
 }
 
+/*
+ * Write an IA and the options it has.
+ */
+int
+write_ia(const struct ia_xx *ia) {
+	struct iasubopt *iasubopt;
+	struct binding *bnd;
+	int i;
+	char addr_buf[sizeof("ffff:ffff:ffff:ffff:ffff:ffff.255.255.255.255")];
+	const char *binding_state;
+	const char *tval;
+	char *s;
+	int fprintf_ret;
+
+	/* 
+	 * If the lease file is corrupt, don't try to write any more 
+	 * leases until we've written a good lease file. 
+	 */
+	if (lease_file_is_corrupt) {
+		if (!new_lease_file()) {
+			return 0;
+		}
+	}
+
+	if (counting) {
+		++count;
+	}
+
+	
+	s = quotify_buf(ia->iaid_duid.data, ia->iaid_duid.len, MDL);
+	if (s == NULL) {
+		goto error_exit;
+	}
+	switch (ia->ia_type) {
+	case D6O_IA_NA:
+		fprintf_ret = fprintf(db_file, "ia-na \"%s\" {\n", s);
+		break;
+	case D6O_IA_TA:
+		fprintf_ret = fprintf(db_file, "ia-ta \"%s\" {\n", s);
+		break;
+	case D6O_IA_PD:
+		fprintf_ret = fprintf(db_file, "ia-pd \"%s\" {\n", s);
+		break;
+	default:
+		log_error("Unknown ia type %u for \"%s\" at %s:%d",
+			  (unsigned)ia->ia_type, s, MDL);
+		fprintf_ret = -1;
+	}
+	dfree(s, MDL);
+	if (fprintf_ret < 0) {
+		goto error_exit;
+	}
+	if (ia->cltt != MIN_TIME) {
+		tval = print_time(ia->cltt);
+		if (tval == NULL) {
+			goto error_exit;
+		}
+		if (fprintf(db_file, "  cltt %s\n", tval) < 0) {
+			goto error_exit;
+		}
+	}
+	for (i=0; i<ia->num_iasubopt; i++) {
+		iasubopt = ia->iasubopt[i];
+
+		inet_ntop(AF_INET6, &iasubopt->addr,
+			  addr_buf, sizeof(addr_buf));
+		if ((ia->ia_type != D6O_IA_PD) &&
+		    (fprintf(db_file, "  iaaddr %s {\n", addr_buf) < 0)) {
+			goto error_exit;
+		}
+		if ((ia->ia_type == D6O_IA_PD) &&
+		    (fprintf(db_file, "  iaprefix %s/%d {\n",
+			     addr_buf, (int)iasubopt->plen) < 0)) {
+			goto error_exit;
+		}
+		if ((iasubopt->state <= 0) || (iasubopt->state > FTS_LAST)) {
+			log_fatal("Unknown iasubopt state %d at %s:%d", 
+				  iasubopt->state, MDL);
+		}
+		binding_state = binding_state_names[iasubopt->state-1];
+		if (fprintf(db_file, "    binding state %s;\n", 
+			    binding_state) < 0) {
+			goto error_exit;
+		}
+		if (fprintf(db_file, "    preferred-life %u;\n",
+			    (unsigned)iasubopt->prefer) < 0) {
+			goto error_exit;
+		}
+		if (fprintf(db_file, "    max-life %u;\n",
+			    (unsigned)iasubopt->valid) < 0) {
+			goto error_exit;
+		}
+
+		/* Note that from here on out, the \n is prepended to the
+		 * next write, rather than appended to the current write.
+		 */
+		if ((iasubopt->state == FTS_ACTIVE) ||
+		    (iasubopt->state == FTS_ABANDONED) ||
+		    (iasubopt->hard_lifetime_end_time != 0)) {
+			tval = print_time(iasubopt->hard_lifetime_end_time);
+		} else {
+			tval = print_time(iasubopt->soft_lifetime_end_time);
+		}
+		if (tval == NULL) {
+			goto error_exit;
+		}
+		if (fprintf(db_file, "    ends %s", tval) < 0) {
+			goto error_exit;
+		}
+
+		/* Write out any binding scopes: note that 'ends' above does
+		 * not have \n on the end!  We want that.
+		 */
+		if (iasubopt->scope != NULL)
+			bnd = iasubopt->scope->bindings;
+		else
+			bnd = NULL;
+
+		for (; bnd != NULL ; bnd = bnd->next) {
+			if (bnd->value == NULL)
+				continue;
+
+			/* We don't do a regular error_exit because the
+			 * lease db is not corrupt in this case.
+			 */
+			if (write_binding_scope(db_file, bnd,
+						"\n    ") != ISC_R_SUCCESS)
+				goto error_exit;
+				
+		}
+
+		if (iasubopt->on_star.on_expiry) {
+			if (fprintf(db_file, "\n    on expiry%s {",
+				    iasubopt->on_star.on_expiry ==
+				    iasubopt->on_star.on_release
+				    ? " or release" : "") < 0)
+				goto error_exit;
+			write_statements(db_file,
+					 iasubopt->on_star.on_expiry, 6);
+			if (fprintf(db_file, "\n    }") < 0) 
+				goto error_exit;
+		}
+
+		if (iasubopt->on_star.on_release &&
+		    iasubopt->on_star.on_release !=
+		    iasubopt->on_star.on_expiry) {
+			if (fprintf(db_file, "\n    on release {") < 0)
+				goto error_exit;
+			write_statements(db_file,
+					 iasubopt->on_star.on_release, 6);
+			if (fprintf(db_file, "\n    }") < 0)
+				goto error_exit;
+		}
+
+		if (fprintf(db_file, "\n  }\n") < 0)
+                        goto error_exit;
+	}
+	if (fprintf(db_file, "}\n\n") < 0)
+                goto error_exit;
+
+	fflush(db_file);
+	return 1;
+
+error_exit:
+	log_info("write_ia: unable to write ia");
+	lease_file_is_corrupt = 1;
+	return 0;
+}
+
+#ifdef DHCPv6
+/*
+ * Put a copy of the server DUID in the leases file.
+ */
+int
+write_server_duid(void) {
+	struct data_string server_duid;
+	char *s;
+	int fprintf_ret;
+
+	/*
+	 * Only write the DUID if it's been set.
+	 */
+	if (!server_duid_isset()) {
+		return 1;
+	}
+
+	/* 
+	 * If the lease file is corrupt, don't try to write any more 
+	 * leases until we've written a good lease file. 
+	 */
+	if (lease_file_is_corrupt) {
+		if (!new_lease_file()) {
+			return 0;
+		}
+	}
+
+	/*
+	 * Get a copy of our server DUID and convert to a quoted string.
+	 */
+	memset(&server_duid, 0, sizeof(server_duid));
+	copy_server_duid(&server_duid, MDL);
+	s = quotify_buf(server_duid.data, server_duid.len, MDL);
+	data_string_forget(&server_duid, MDL);
+	if (s == NULL) {
+		goto error_exit;
+	}
+
+	/*
+	 * Write to the leases file.
+	 */
+	fprintf_ret = fprintf(db_file, "server-duid \"%s\";\n\n", s);
+	dfree(s, MDL);
+	if (fprintf_ret < 0) {
+		goto error_exit;
+	}
+
+	/*
+	 * Check if we actually managed to write.
+	 */
+	fflush(db_file);
+	return 1;
+
+error_exit:
+	log_info("write_server_duid: unable to write server-duid");
+	lease_file_is_corrupt = 1;
+	return 0;
+}
+#endif /* DHCPv6 */
+
 #if defined (FAILOVER_PROTOCOL)
 int write_failover_state (dhcp_failover_state_t *state)
 {
-	struct tm *t;
 	int errors = 0;
 	const char *tval;
 
@@ -696,8 +950,6 @@ void write_billing_classes ()
 {
 	struct collection *lp;
 	struct class *cp;
-	struct hash_bucket *bp;
-	int i;
 
 	for (lp = collections; lp; lp = lp -> next) {
 	    for (cp = lp -> classes; cp; cp = cp -> nic) {
@@ -714,7 +966,6 @@ int write_billing_class (class)
 	struct class *class;
 {
 	int errors = 0;
-	int i;
 
 	if (lease_file_is_corrupt)
 		if (!new_lease_file ())
@@ -757,23 +1008,39 @@ int commit_leases ()
 	   We need to do this even if we're rewriting the file below,
 	   just in case the rewrite fails. */
 	if (fflush (db_file) == EOF) {
-		log_info ("commit_leases: unable to commit: %m");
-		return 0;
+		log_info("commit_leases: unable to commit, fflush(): %m");
+		return (0);
 	}
-	if (fsync (fileno (db_file)) < 0) {
-		log_info ("commit_leases: unable to commit: %m");
-		return 0;
+	if ((dont_use_fsync == 0) &&
+	    (fsync(fileno (db_file)) < 0)) {
+		log_info ("commit_leases: unable to commit, fsync(): %m");
+		return (0);
 	}
 
 	/* If we haven't rewritten the lease database in over an
 	   hour, rewrite it now.  (The length of time should probably
 	   be configurable. */
-	if (count && cur_time - write_time > 3600) {
+	if (count && cur_time - write_time > LEASE_REWRITE_PERIOD) {
 		count = 0;
 		write_time = cur_time;
-		new_lease_file ();
+		new_lease_file();
 	}
-	return 1;
+	return (1);
+}
+
+/*
+ * rewrite the lease file about once an hour
+ * This is meant as a quick patch for ticket 24887.  It allows
+ * us to rotate the v6 lease file without adding too many fsync()
+ * calls.  In the future wes should revisit this area and add
+ * something similar to the delayed ack code for v4.
+ */
+int commit_leases_timed()
+{
+	if ((count != 0) && (cur_time - write_time > LEASE_REWRITE_PERIOD)) {
+		return (commit_leases());
+	}
+	return (1);
 }
 
 void db_startup (testp)
@@ -787,7 +1054,11 @@ void db_startup (testp)
 		/* Read in the existing lease file... */
 		status = read_conf_file (path_dhcpd_db,
 					 (struct group *)0, 0, 1);
-		/* XXX ignore status? */
+		if (status != ISC_R_SUCCESS) {
+			/* XXX ignore status? */
+			;
+		}
+
 #if defined (TRACING)
 	}
 #endif
@@ -810,7 +1081,7 @@ void db_startup (testp)
 			write_time = cur_time;
 		else
 #endif
-			GET_TIME (&write_time);
+			time(&write_time);
 		new_lease_file ();
 	}
 
@@ -835,7 +1106,7 @@ int new_lease_file ()
 	FILE *new_db_file;
 
 	/* Make a temporary lease file... */
-	GET_TIME (&t);
+	time(&t);
 
 	db_validity = lease_file_is_corrupt;
 
@@ -854,6 +1125,22 @@ int new_lease_file ()
 		log_error ("Can't create new lease file: %m");
 		return 0;
 	}
+
+#if defined (PARANOIA)
+	/*
+	 * If we are currently root and plan to change the
+	 * uid and gid change the file information so we
+	 * can manipulate it later, after we've changed
+	 * our group and user (that is dropped privileges.)
+	 */
+	if ((set_uid != 0) && (geteuid() == 0) &&
+	    (set_gid != 0) && (getegid() == 0)) {
+		if (fchown(db_fd, set_uid, set_gid)) {
+			log_fatal ("Can't chown new lease file: %m");
+		}
+	}
+#endif /* PARANOIA */
+
 	if ((new_db_file = fdopen(db_fd, "w")) == NULL) {
 		log_error("Can't fdopen new lease file: %m");
 		close(db_fd);
@@ -872,7 +1159,7 @@ int new_lease_file ()
 		goto fail;
 
 	fprintf (db_file, "# This lease file was written by isc-dhcp-%s\n\n",
-		 DHCP_VERSION);
+		 PACKAGE_VERSION);
 	if (errno)
 		goto fail;
 
@@ -932,7 +1219,7 @@ int new_lease_file ()
       fail:
 	lease_file_is_corrupt = db_validity;
       fdfail:
-	unlink (newfname);
+	(void)unlink (newfname);
 	return 0;
 }
 
